==> clean_android_design-0.0.191.tar.gz: lib/src/fields/selectable_list_field_widget.dart <==
import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';

class SelectableListFieldWidget<T extends IdItemInterface>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(List<DocumentReference> selectedItemRefs)
      saveSelectedValues;
  final List<DocumentReference> Function() getSelectedValues;

  final bool Function(T item) selectableItemsLambda;
  final Future<bool> Function(T item) selectableItemsLambdaAsync;
  final ISelectableListItem Function(BuildContext context, T item) itemBuilder;
  final Comparable Function(T item) sortProperty;

  final bool isEditable;

  final Widget noItemWidget;
  final Widget noItemListItem;

  final Widget topWidget;
  final Widget title;

  final ForceUpdateKey activateKeyOnChange;
  final ItemSectionExtendedController controller;

  final DocumentReference Function(T item) getReferenceFromItem;
  final List<T> Function() getFullItemList;

  const SelectableListFieldWidget({
    Key key,
    this.saveSelectedValues,
    @required this.getSelectedValues,
    this.selectableItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemListItem,
    this.sortProperty,
    this.topWidget,
    this.title,
    this.activateKeyOnChange,
    @required this.isEditable,
    this.controller,
    @required this.getReferenceFromItem,
    @required this.getFullItemList,
    this.selectableItemsLambdaAsync,
  })  : isFirst = false,
        isLast = false,
        assert(getSelectedValues != null),
        assert(!isEditable || saveSelectedValues != null),
        assert(noItemListItem == null || noItemWidget == null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<T> createState() =>
      _SelectableListFieldWidgetState<T>();
}

class _SelectableListFieldWidgetState<T extends IdItemInterface>
    extends State<SelectableListFieldWidget<T>> {
  List<T> selectableItems;
  bool loadingData;

  void loadSelectableItems() async {
    loadingData = true;

    selectableItems = <T>[];

    for (var item in widget.getFullItemList()) {
      if ((widget.selectableItemsLambda == null &&
              widget.selectableItemsLambdaAsync == null) ||
          (widget.selectableItemsLambda != null &&
              widget.selectableItemsLambda(item)) ||
          (widget.selectableItemsLambdaAsync != null &&
              await widget.selectableItemsLambdaAsync(item))) {
        selectableItems.add(item);
      }
    }

    if (widget.sortProperty != null)
      selectableItems.sort((i1, i2) =>
          widget.sortProperty(i1).compareTo(widget.sortProperty(i2)));

    setState(() => loadingData = false);
  }

  @override
  void initState() {
    super.initState();

    loadSelectableItems();
  }

  @override
  Widget build(BuildContext context) {
    if (loadingData)
      return _buildWaitingWidget();
    else
      return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _buildWaitingWidget() {
    return Padding(
      padding: EdgeInsets.all(30),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
            width: 70,
            height: 70,
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation<Color>(
                  CleanAndroidDesignDefaultColors.defaultColor),
            ),
          ),
        ],
      ),
    );
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? selectableItems
        : selectableItems
            .where(
                (e) => widget.getSelectedValues().any((ref) => ref.id == e.id))
            .toList();

    if (items.length == 0 && widget.noItemListItem == null)
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        items.length != 0
            ? ItemSectionWidget.builder(
                title: widget.title,
                controller: widget.controller,
                itemBuilder: (context, index) =>
                    widget.itemBuilder(context, items[index]).copyWithSelection(
                          isSelected: isEditable
                              ? widget
                                  .getSelectedValues()
                                  .any((ref) => ref.id == items[index].id)
                              : false,
                          onTapList: isEditable
                              ? () {
                                  _changeSelectionFor(items[index]);
                                  return false;
                                }
                              : null,
                        ),
                itemCount: items.length,
              )
            : (widget.noItemListItem == null
                ? Container()
                : ItemSectionWidget(
                    title: widget.title,
                    controller: widget.controller,
                    children: [widget.noItemListItem],
                  )),
      ],
    );
  }

  _changeSelectionFor(T item) {
    var selectedItems = widget.getSelectedValues();
    if (selectedItems.any((ref) => ref.id == item.id))
      selectedItems.removeWhere((ref) => ref.id == item.id);
    else
      selectedItems.add(widget.getReferenceFromItem(item));

    setState(() => widget.saveSelectedValues(selectedItems));
    widget.activateKeyOnChange?.update();
  }
}

==> clean_android_design-0.1.0.tar.gz: lib/src/fields/selectable_list_field_widget.dart <==
import 'package:clean_android_design/clean_android_design.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';

class SelectableListFieldWidget<T extends IdItemInterface>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(List<DocumentReference> selectedItemRefs)?
      saveSelectedValues;
  final List<DocumentReference> Function() getSelectedValues;

  final bool Function(T item)? selectableItemsLambda;
  final Future<bool> Function(T item)? selectableItemsLambdaAsync;
  final ISelectableListItem Function(BuildContext context, T item)? itemBuilder;
  final Comparable Function(T item)? sortProperty;

  final bool isEditable;

  final Widget? noItemWidget;
  final Widget? noItemListItem;

  final Widget? topWidget;
  final Widget? title;

  final ForceUpdateKey? activateKeyOnChange;
  final ItemSectionExtendedController? controller;

  final DocumentReference Function(T item) getReferenceFromItem;
  final List<T> Function() getFullItemList;

  const SelectableListFieldWidget({
    Key? key,
    this.saveSelectedValues,
    required this.getSelectedValues,
    this.selectableItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemListItem,
    this.sortProperty,
    this.topWidget,
    this.title,
    this.activateKeyOnChange,
    required this.isEditable,
    this.controller,
    required this.getReferenceFromItem,
    required this.getFullItemList,
    this.selectableItemsLambdaAsync,
  })  : isFirst = false,
        isLast = false,
        assert(!isEditable || saveSelectedValues != null),
        assert(noItemListItem == null || noItemWidget == null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<T> createState() =>
      _SelectableListFieldWidgetState<T>();
}

class _SelectableListFieldWidgetState<T extends IdItemInterface>
    extends State<SelectableListFieldWidget<T>> {
  late List<T> selectableItems;
  late bool loadingData;

  void loadSelectableItems() async {
    loadingData = true;

    selectableItems = <T>[];

    for (var item in widget.getFullItemList()) {
      if ((widget.selectableItemsLambda == null &&
              widget.selectableItemsLambdaAsync == null) ||
          (widget.selectableItemsLambda != null &&
              widget.selectableItemsLambda!(item)) ||
          (widget.selectableItemsLambdaAsync != null &&
              await widget.selectableItemsLambdaAsync!(item))) {
        selectableItems.add(item);
      }
    }

    if (widget.sortProperty != null)
      selectableItems.sort((i1, i2) =>
          widget.sortProperty!(i1).compareTo(widget.sortProperty!(i2)));

    setState(() => loadingData = false);
  }

  @override
  void initState() {
    super.initState();

    loadSelectableItems();
  }

  @override
  Widget build(BuildContext context) {
    if (loadingData)
      return _buildWaitingWidget();
    else
      return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _buildWaitingWidget() {
    return Padding(
      padding: EdgeInsets.all(30),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Container(
            width: 70,
            height: 70,
            child: CircularProgressIndicator(
              valueColor: AlwaysStoppedAnimation<Color?>(
                  CleanAndroidDesignDefaultColors.defaultColor),
            ),
          ),
        ],
      ),
    );
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? selectableItems
        : selectableItems
            .where(
                (e) => widget.getSelectedValues().any((ref) => ref.id == e.id))
            .toList();

    if (items.length == 0 && widget.noItemListItem == null)
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        items.length != 0
            ? ItemSectionWidget.builder(
                title: widget.title,
                controller: widget.controller,
                itemBuilder: (context, index) => widget.itemBuilder!
                        (context, items[index])
                    .copyWithSelection(
                  isSelected: isEditable
                      ? widget
                          .getSelectedValues()
                          .any((ref) => ref.id == items[index].id)
                      : false,
                  onTapList: isEditable
                      ? () {
                          _changeSelectionFor(items[index]);
                          return false;
                        }
                      : null,
                ),
                itemCount: items.length,
              )
            : (widget.noItemListItem == null
                ? Container()
                : ItemSectionWidget(
                    title: widget.title,
                    controller: widget.controller,
                    children: [widget.noItemListItem],
                  )),
      ],
    );
  }

  _changeSelectionFor(T item) {
    var selectedItems = widget.getSelectedValues();
    if (selectedItems.any((ref) => ref.id == item.id))
      selectedItems.removeWhere((ref) => ref.id == item.id);
    else
      selectedItems.add(widget.getReferenceFromItem(item));

    setState(() => widget.saveSelectedValues!(selectedItems));
    widget.activateKeyOnChange?.update();
  }
}
